# Copyright (C) 2016, A10 Networks Inc. All rights reserved.

import logging

from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import messages

# from openstack_dashboard import api
from a10_horizon.dashboard.api import certificates as api

LOG = logging.getLogger(__name__)


class UpdateCertificate(forms.SelfHandlingForm):
    id = forms.CharField(label=_("ID"), widget=forms.TextInput(
                         attrs={'readonly': 'readonly'}))
    name = forms.CharField(label=_("Name"), min_length=1, max_length=255,
                           required=True)
    description = forms.CharField(label=_("Description"), min_length=1,
                                  max_length=255, required=True)
    cert_data = forms.CharField(label=_("Data"), min_length=1, max_length=1024,
                                required=True)
    key_data = forms.CharField(
        label=_("Key Data"), min_length=1, max_length=8000,
        widget=forms.Textarea(attrs={'cols': 80, 'rows': 15}), required=True)
    password = forms.CharField(
        label=_("Password"), min_length=1, max_length=1024,
        widget=forms.PasswordInput(render_value=False), required=False)
    intermediate_data = forms.CharField(
        label=_("Intermediate Certificate Data"), min_length=1, max_length=8000,
        widget=forms.Textarea(attrs={'cols': 80, 'rows': 15}), required=False)

    failure_url = 'horizon:project:a10ssl:index'

    def handle(self, request, context):
        LOG.debug("UpdateCertificate:handle(): context" % context)
        try:
            certificate = api.certificate_update(request, **context)
            msg = _('Certificate {0} was successfully updated.').format(context['id'])
            LOG.debug(msg)
            messages.success(request, msg)
            return certificate
        except Exception:
            msg = _('Failed to update certificate %s') % context['id']
            LOG.exception(msg)
            redirect = reverse(self.failure_url)
            exceptions.handle(request, msg, redirect=redirect)



# Copyright (C) 2014-2016, A10 Networks Inc. All rights reserved.

OP_DICT = {
    ">": "greater than",
    "<": "less than",
    ">=": "greater than or equal to",
    "<=": "less than or equal to",
}

AGG_DICT = {
    "avg": "average",
    "sum": "sum",
    "min": "minimum",
    "max": "maximum"
}

UNIT_DICT = {
    "count": "",
    "percentage": "%",
    "bytes": "bytes",
}

MEASUREMENTS = [
    "connections",
    "cpu",
    "interface",
    "memory"
]

VALID_MEASUREMENT_UNITS = {
    "connections": ["count"],
    "cpu": ["percentage"],
    "interface": ["percentage", "count"],
    "memory": ["bytes", "percentage"],
}



# Copyright (C) 2014-2016, A10 Networks Inc. All rights reserved.

import logging

from django.core.urlresolvers import reverse_lazy
from django.shortcuts import redirect
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import tables
from horizon import workflows

# a10_horizon.dashboard.api.client.Client extends neutron.api.client.Client
from a10_neutronclient.resources import a10_scaling_group as scaling_resources
from a10_horizon.dashboard.api import scaling as api


LOG = logging.getLogger(__name__)


class AddPolicyAction(workflows.Action):
    name = forms.CharField(label=_("Name"), min_length=1, max_length=255,
                           required=True)
    description = forms.CharField(label=_("Description"), min_length=1,
                                  max_length=255, required=False)
    # TODO(mdurrant) - Get min/max cooldown/instances from config?
    cooldown = forms.IntegerField(label=_("Cooldown Period in Seconds"),
                                  min_value=1,
                                  max_value=65536,
                                  required=True)
    min_instances = forms.IntegerField(label=_("Minimum Instances"),
                                       min_value=1,
                                       required=True)
    max_instances = forms.IntegerField(label=_("Maximum Instances"),
                                       required=False)

    class Meta(object):
        name = _("Create New Scaling Policy")
        # TODO(mdurrant) - Add a10-specific permissions
        permissions = ("openstack.services.network", )
        help_text = _("Specify the details for the scaling policy below")


class AddPolicyStep(workflows.Step):
    action_class = AddPolicyAction
    contributes = ("name", "description", "cooldown", "min_instances", "max_instances")


class AddPolicyWorkflow(workflows.Workflow):
    slug = "addscalingpolicy"
    name = _("Add Scaling Policy")
    default_steps = (AddPolicyStep, )
    success_url = "horizon:project:a10scaling:index"
    finalize_button_name = "Create Scaling Policy"

    def handle(self, request, context):
        success = True
        try:
            api.create_a10_scaling_policy(request, **context)
        except Exception as ex:
            LOG.exception(ex)
            exceptions.handle(request, _("Unable to create scaling policy"))
        return success


class DeletePolicyWorkflow(workflows.Workflow):
    name = "deletescalingpolicy"
    action_present = _("Delete")
    action_past = _("Scheduled deletion of %(data_type)s")
    data_type_singular = _("Scaling Policy")
    data_type_plural = _("Scaling Policies")

    def handle(self, data_table, request, object_ids):
        for obj_id in object_ids:
            try:
                api.delete_a10_scaling_policy(request, obj_id)
                redirect_url = 'horizon:project:a10scaling:index'
            except Exception as ex:
                LOG.exception(ex)
                exceptions.handle(request, _("Unable to delete scaling policy"))

        return redirect(redirect_url)


class AddActionAction(workflows.Action):
    name = forms.CharField(label=_("Name"), min_length=1, max_length=255,
                           required=True)
    description = forms.CharField(label=_("Description"), min_length=1,
                                  max_length=255, required=False)
    action = forms.ChoiceField(label=_("Action"),
                               choices=map(lambda x: (x, x), scaling_resources.ACTIONS),
                               required=True)
    amount = forms.IntegerField(label=_("Amount"), required=True)

    class Meta(object):
        name = _("Create New Scaling Action")
        # TODO(mdurrant) - Add a10-specific permissions
        permissions = ("openstack.services.network", )
        help_text = _("Specify the details for the scaling action below")


class AddActionStep(workflows.Step):
    action_class = AddActionAction
    contributes = ("name", "description", "action", "amount",)


class AddActionWorkflow(workflows.Workflow):
    slug = "addscalingaction"
    name = _("Add Scaling Action")
    default_steps = (AddActionStep, )
    success_url = "horizon:project:a10scaling:index"
    finalize_button_name = "Create Action"

    def handle(self, request, context):
        try:
            api.create_a10_scaling_action(request, **context)
        except Exception as ex:
            LOG.exception(ex)
            exceptions.handle(request, _("Unable to delete scaling action"))
        return redirect(self.success_url)


class DeleteScalingActionAction(tables.BatchAction):
    name = "deletescalingaction"
    action_present = _("Delete")
    action_past = _("Scheduled deletion of %(data_type)s")
    data_type_singular = _("Scaling Action")
    data_type_plural = _("Scaling Policies")
    success_url = "horizon:project:a10scaling:index"

    def handle(self, data_table, request, object_ids):
        for obj_id in object_ids:
            try:
                api.delete_a10_scaling_action(request, obj_id)
            except Exception as ex:
                LOG.exception(ex)
                exceptions.handle(request, _("Unable to delete scaling alarms"))

        return redirect(self.success_url)


class AddAlarmAction(workflows.Action):
    name = forms.CharField(label=_("Name"), min_length=1, max_length=255,
                           required=True)
    description = forms.CharField(label=_("Description"), min_length=1,
                                  max_length=255, required=False)
    aggregation = forms.DynamicChoiceField(label=_("Aggregation"),
                                           choices=map(lambda x: (x, x),
                                                       scaling_resources.ALARM_AGGREGATIONS),
                                           required=True)
    measurement = forms.ChoiceField(label=_("Measurement"),
                                    choices=map(lambda x: (x, x),
                                                scaling_resources.ALARM_MEASUREMENTS),
                                    required=True)
    operator = forms.ChoiceField(label=_("Operator"),
                                 choices=map(lambda x: (x, x),
                                             scaling_resources.ALARM_OPERATORS),
                                 required=True,)
    threshold = forms.FloatField(label=_("Threshold"),
                                 required=True)
    unit = forms.DynamicChoiceField(label=_("Unit"),
                                    choices=map(lambda x: (x, x),
                                                scaling_resources.ALARM_UNITS),
                                    required=True)
    period = forms.IntegerField(label=_("Period"),
                                required=True)
    period_unit = forms.ChoiceField(label=_("Period Unit"),
                                    choices=map(lambda x: (x, x),
                                                scaling_resources.ALARM_PERIOD_UNITS),
                                    required=True)

    class Meta(object):
        name = _("Create New Scaling Alarm")
        # TODO(mdurrant) - Add a10-specific permissions
        permissions = ("openstack.services.network", )
        help_text = _("Specify the details for the scaling action below")


class AddAlarmStep(workflows.Step):
    action_class = AddAlarmAction
    contributes = ("name", "description", "aggregation", "measurement", "operator",
                   "threshold", "unit", "period", "period_unit")


class AddScalingAlarmWorkflow(workflows.Workflow):
    slug = "addscalingalarm"
    name = _("Add Scaling Alarm")
    default_steps = (AddAlarmStep, )
    success_url = "horizon:project:a10scaling:index"
    finalize_button_name = "Create Alarm"

    def handle(self, request, context):
        success = True
        try:
            api.create_a10_scaling_alarm(request, **context)
        except Exception as ex:
            LOG.exception(ex)
            exceptions.handle(request, _("Unable to create scaling alarms"))
        return success


class DeleteScalingAlarmAction(tables.BatchAction):
    name = "deletescalingalarm"
    action_present = _("Delete")
    action_past = _("Scheduled deletion of %(data_type)s")
    data_type_singular = _("Scaling Alarm")
    data_type_plural = _("Scaling Policies")

    def handle(self, data_table, request, object_ids):
        for obj_id in object_ids:
            try:
                api.delete_a10_scaling_alarm(request, obj_id)
            except Exception as ex:
                LOG.exception(ex)
                exceptions.handle(request, _("Unable to delete scaling alarms"))

        return redirect("horizon:project:a10scaling:index")


class AddReactionAction(workflows.Action):
    scaling_policy_id = forms.Field(widget=forms.HiddenInput, initial="")
    alarm_id = forms.ChoiceField(label=_("Alarm"))
    action_id = forms.ChoiceField(label=_("Action"))
    detail_url = "horizon:project:a10scaling:scalingpolicydetail"

    def __init__(self, request, *args, **kwargs):
        alarms = []
        actions = []
        # Make it so we don't have to pull this out of the tuples arg.
        policy_id = args[0]["scaling_policy_id"]

        alarm_choices = [('', _('Select an alarm'))]
        action_choices = [('', _('Select an action'))]
        super(AddReactionAction, self).__init__(request, *args, **kwargs)

        try:
            alarms = api.get_a10_scaling_alarms(request)
        except Exception as ex:
            LOG.exception(ex)
            exceptions.handle(request, _("Unable to retrieve scaling alarms"))

        try:
            actions = api.get_a10_scaling_actions(request)
        except Exception as ex:
            LOG.exception(ex)
            exceptions.handle(request, _("Unable to retrieve scaling alarms"))

        choice_map = lambda x: (x["id"], x["name"])

        map(alarm_choices.append, map(choice_map, alarms))
        map(action_choices.append, map(choice_map, actions))

        self.fields["scaling_policy_id"].value = policy_id
        self.fields["alarm_id"].choices = alarm_choices
        self.fields["action_id"].choices = action_choices
        self.success_url = reverse_lazy(self.detail_url,
                                        kwargs={"scaling_policy_id": policy_id})

    def _convert_reactions(self, reactions):
        """Extracts action/alarm IDs from reactions retrieved via Neutron API"""
        return map(lambda x: {"alarm_id": x["alarm_id"],
                              "action_id": x["action_id"]}, reactions)

    def handle(self, request, context):
        policy_id = context["scaling_policy_id"]

        try:
            policy = api.get_a10_scaling_policy(request, policy_id)
            reactions = self._convert_reactions(policy.reactions)
        except Exception as ex:
            LOG.exception(ex)
            exceptions.handle(request, _("Unable to retrieve existing policy."))
            return False

        try:
            # Extract the IDs from the reactions since that's all we need
            map(reactions.append, self._convert_reactions([context]))

            # convert to dictioary for API
            policy = policy.to_dict()
            policy["reactions"] = reactions

            # read only attribute, this should probably go in clean()
            del policy["tenant_id"]

            api.update_a10_scaling_policy(request, **policy)

        except Exception as ex:
            LOG.exception(ex)
            exceptions.handle(request, _("Unable to update policy."))
            return False

        return True

    class Meta(object):
        name = _("Add Scaling Reaction")
        permissions = ('openstack.services.network',)
        help_text = _("Specify reactions for the scaling policy")


class AddReactionStep(workflows.Step):
    action_class = AddReactionAction
    contributes = ("scaling_policy_id", "alarm_id", "action_id")


class AddReactionWorkflow(workflows.Workflow):
    slug = "addreaction"
    name = _("Add Scaling Reaction")
    default_steps = (AddReactionStep, )
    finalize_button_name = "Create Reaction"
    redirect_url = "horizon:project:a10scaling:index"
    detail_url = "horizon:project:a10scaling:scalingpolicydetail"
    success_message = _("Created Scaling Reaction")  # noqa
    failure_message = _("Unable to create scaling reaction")  # noqa

    def handle(self, request, context):
        success = True
        # Get the existing policy
        try:
            scaling_policy_id = context["scaling_policy_id"]
            self.success_url = reverse_lazy(self.detail_url,
                                            kwargs={"scaling_policy_id": scaling_policy_id})
            policy = api.get_a10_scaling_policy(request, scaling_policy_id)
            context["scaling_policy"] = policy
            success = True
        except Exception as ex:
            LOG.exception(ex)
            exceptions.handle(request, _("Unable to add scaling reaction"))
        # Re-assign order as needed.

        return success



# Copyright (C) 2016 A10 Networks Inc. All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from keystoneclient.auth.identity import generic as auth_plugin
from keystoneclient import session as keystone_session
from openstack_dashboard.api import base


def token_for(request):
    auth_url = base.url_for(request, 'identity')
    auth_token = request.user.token.unscoped_token
    project_name = request.user.token.project["name"]
    return auth_plugin.Token(token=auth_token,
                             project_name=project_name,
                             auth_url=auth_url)


def session_for(request):
    token = token_for(request)
    session = keystone_session.Session(auth=token)
    return session


def project_id_for(request):
    return request.user.token.project["id"]



# Copyright (C) 2014-2016, A10 Networks Inc. All rights reserved.

import logging

from django.core.urlresolvers import reverse
from django.core.urlresolvers import reverse_lazy
from django.utils.translation import ugettext_lazy as _
from horizon import tables

LOG = logging.getLogger(__name__)


class AddCertificateLink(tables.LinkAction):
    name = "addcertificate"
    verbose_name = _("Add Certificate")
    url = "horizon:project:a10ssl:addcertificate"
    classes = ("ajax-modal",)
    icon = "plus"
    policy_rules = (("network", "create_certificate"),)  # FIXME(traff)


class DeleteCertificateLink(tables.DeleteAction):
    name = "deletecertificate"
    action_present = _("Delete")
    action_past = _("Scheduled deletion of %(data_type)s")
    data_type_singular = _("Certificate")
    data_type_plural = _("Certificates")

    # def allowed(self, request, datum=None):
    #     is_allowed = (datum and datum.id)
    #     LOG.debug("DeleteCertificateLink:allowed(): datum=%s,is_allowed=%s" % (datum, is_allowed))
    #     return True


class DeleteCertificateBindingLink(tables.DeleteAction):
    name = "deletecertificatebinding"
    action_present = _("Delete")
    action_past = _("Scheduled deletion of %(data_type)s")
    data_type_singular = _("Certificate Association")
    data_type_plural = _("Certificate Associations")

    # def allowed(self, request, datum=None):
    #     is_allowed = (datum and datum.id)
    #     LOG.debug("DeleteCertificateBindingLink:allowed(): datum=%s,is_allowed=%s").format(
    #         datum, is_allowed)
    #     return True


class AddCertificateBindingLink(tables.LinkAction):
    name = "addcertificatebinding"
    verbose_name = _("Add Certificate Association")
    url = "horizon:project:a10ssl:addcertificatebinding"
    classes = ("ajax-modal",)
    icon = "plus"
    policy_rules = (("network", "create_certificate_binding"),)  # FIXME(traff)


class UpdateCertificateLink(tables.LinkAction):
    name = "updatecertificate"
    verbose_name = _("Edit Certificate")
    classes = ("ajax-modal", "btn-update",)

    def get_link_url(self, certificate):
        base_url = reverse_lazy("horizon:project:a10ssl:updatecertificate",
                                kwargs={'certificate_id': certificate.id})
        return base_url


class CertificatesTable(tables.DataTable):
    id = tables.Column("id", verbose_name=_("ID"), hidden=True)
    name = tables.Column("name", verbose_name=_("Name"))
    description = tables.Column("description", verbose_name=_("Description"))

    class Meta(object):
        name = "certificatestable"
        verbose_name = _("Certificates")
        table_actions = (AddCertificateLink, DeleteCertificateLink)
        row_actions = (UpdateCertificateLink, )


class CertificateBindingsTable(tables.DataTable):
    id = tables.Column("id", verbose_name=_("ID"), hidden=True)
    vip_id = tables.Column("vip_id", verbose_name=_("VIP ID"), hidden=True)
    certificate_id = tables.Column("certificate_id", verbose_name=_("Certificate ID"), hidden=True)
    vip_name = tables.Column("vip_name", verbose_name=_("VIP"))
    certificate_name = tables.Column("certificate_name", verbose_name=_("Certificate"))

    class Meta(object):
        name = "certificatebindingtable"
        verbose_name = _("Certificate Associations")
        table_actions = (AddCertificateBindingLink, DeleteCertificateBindingLink)
        row_actions = ()



# Copyright 2015 A10 Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.utils.translation import ugettext_lazy as _

from horizon import tabs
from horizon import workflows

import a10_horizon.dashboard.a10networks.a10appliances.tabs as p_tabs
import a10_horizon.dashboard.a10networks.a10appliances.workflows as p_workflows


class IndexView(tabs.TabbedTableView):
    tab_group_class = p_tabs.A10Tabs
    template_name = "appliances_tabs.html"




# Copyright (C) 2014-2015, A10 Networks Inc. All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging

# from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ungettext_lazy

from horizon import tables


LOG = logging.getLogger(__name__)


class A10ApplianceTable(tables.DataTable):
    id = tables.Column("id", verbose_name=_("ID"), hidden=True)
    name = tables.Column("name", verbose_name=_("Hostname"), hidden=False)
    ip = tables.Column("host", verbose_name="Management IP")
    api_ver = tables.Column("api_version", verbose_name="API Version")
    nova_instance_id = tables.Column("nova_instance_id", hidden=True)

    class Meta(object):
        name = "a10appliancestable"
        verbose_name = _("A10 Appliances")
        table_actions = ()
        row_actions = ()


class A10DeviceInstanceTable(tables.DataTable):
    id = tables.Column("id", verbose_name=_("ID"), hidden=True)
    tenant_id = tables.Column("tenant_id", verbose_name=_("Tenant ID"), hidden=True)
    ip_address = tables.Column("ip_address", verbose_name=_("IP Address"), hidden=True)
    nova_instance_id = tables.Column("nova_instance_id", verbose_name=_("Nova Instance ID"),
                                     hidden=True)

    class Meta(object):
        name = "a10deviceinstancetable"
        verbose_name = "a10deviceinstancetable"
        table_actions = ()
        row_actions = ()



# Copyright (C) 2014-2015, A10 Networks Inc. All rights reserved.

import logging

from django.utils.translation import ugettext_lazy as _

from a10_neutron_lbaas import a10_config
from horizon import tabs
from horizon import exceptions

import tables
from a10_horizon.dashboard.api import certificates as cert_api


LOG = LOG = logging.getLogger(__name__)


class CertificatesTab(tabs.TableTab):
    table_classes = (tables.CertificatesTable,)
    name = _("Certificates")
    slug = "certificates"
    template_name = "certificates/_certificates_tab.html"
    preload = False

    def get_certificatestable_data(self):
        try:
            tenant_id = self.request.user.tenant_id
            certificates = []
            certificates = cert_api.certificate_list(self.tab_group.request,
                                                     tenant_id=tenant_id)
        except Exception as ex:
            certificates = []
            exceptions.handle(self.tab_group.request,
                              _('Unable to retrieve certificate list.'))
            LOG.error("Could not retrieve certificate list. ERROR=%s" % ex)
        return certificates


class CertificateBindingsTab(tabs.TableTab):
    table_classes = (tables.CertificateBindingsTable,)
    name = _("Certificate Associations")
    slug = "certificatebindings"
    template_name = "certificates/_certificatebindings_tab.html"

    def get_certificatebindingtable_data(self):
        try:
            tenant_id = self.request.user.tenant_id
            bindings = []
            bindings = cert_api.certificate_bindings_list(self.tab_group.request,
                                                          tenant_id=tenant_id)
        except Exception as ex:
            bindings = []
            exceptions.handle(self.tab_group.request, _("Unable to retrieve certificate "
                                                        "associations list."))
            LOG.error("Could not retrieve certificate associations list. ERROR=%s" % ex)

        return bindings


class A10SSLTabs(tabs.TabGroup):
    slug = "a10ssltabs"
    tabs = (CertificatesTab,
            CertificateBindingsTab
            )
    sticky = True



# Copyright (C) 2014-2015, A10 Networks Inc. All rights reserved.

from django.core.urlresolvers import reverse_lazy
from django.shortcuts import redirect
from django.utils.translation import ugettext_lazy as _
import logging

from horizon import exceptions
from horizon import forms
from horizon.utils import memoized
from horizon import messages
from horizon import tabs
from horizon import tables
from horizon import workflows
import re

from a10_horizon.dashboard.api import scaling as api

import forms as project_forms
import tables as project_tables
import tabs as project_tabs
import workflows as project_workflows


LOG = logging.getLogger(__name__)

ACTION = "action"
NOUN = "noun"
PLURAL = "plural"

URL_PREFIX = "horizon:project:a10scaling:"


class IndexView(tabs.TabView):
    tab_group_class = (project_tabs.A10ScalingTabs)
    template_name = 'scaling_tabs.html'
    form_url = reverse_lazy(URL_PREFIX + "index")

    delete_actions = {
        "scalingpolicy": {
            ACTION: api.delete_a10_scaling_policy,
            NOUN: "Scaling Policy",
            PLURAL: "Scaling Policies",
        },
        "scalingaction": {
            ACTION: api.delete_a10_scaling_action,
            NOUN: "Scaling Action",
            PLURAL: "Scaling Actions",
        },
        "scalingalarm": {
            ACTION: api.delete_a10_scaling_alarm,
            NOUN: "Scaling Alarm",
            PLURAL: "Scaling Alarms",
        },
    }

    def post(self, request, *args, **kwargs):
        obj_ids = request.POST.getlist('object_ids')
        action = request.POST['action']
        m = re.search('.delete([a-z]+)', action).group(1)
        if obj_ids == []:
            obj_ids.append(re.search('([0-9a-z-]+)$', action).group(1))

        if m in self.delete_actions:
            delete_action = self.delete_actions[m]
            for obj_id in obj_ids:
                success_msg = "Deleted {0} {1}".format(delete_action[NOUN], obj_id)
                failure_msg = "Unable to delete {0} {1}".format(delete_action[NOUN], obj_id)

                try:
                    delete_action[ACTION](request, obj_id)
                    messages.success(request, success_msg)
                except Exception as ex:
                    exceptions.handle(request, failure_msg)
                    LOG.exception(ex)

        return self.get(request, *args, **kwargs)


class AddPolicyView(workflows.WorkflowView):
    name = _("Create Scaling Policy")
    workflow_class = project_workflows.AddPolicyWorkflow
    success_url = reverse_lazy(URL_PREFIX + "index")


class UpdatePolicyView(forms.views.ModalFormView):
    name = _("Update Scaling Policy")
    form_class = project_forms.UpdatePolicy
    context_object_name = "scaling_policy"
    success_url = reverse_lazy(URL_PREFIX + "index")
    template_name = "update.html"
    page_title = "Update Scaling Policy"

    def get_context_data(self, **kwargs):
        context = super(UpdatePolicyView, self).get_context_data(**kwargs)
        return context

    @memoized.memoized_method
    def _get_object(self, *args, **kwargs):
        id = self.kwargs['scaling_policy_id']
        self.submit_url = reverse_lazy(URL_PREFIX + "updatescalingpolicy",
                                       kwargs={"scaling_policy_id": id})
        if id:
            try:
                return api.get_a10_scaling_policy(self.request, id)
            except Exception as ex:
                redirect = self.success_url
                msg = _("Unable to retrieve scaling policy: %s") % ex
                exceptions.handle(self.request, msg, redirect=redirect)

    def get_initial(self):
        rv = self._get_object()
        return rv


class PolicyDetailView(tables.MultiTableView):
    name = _("Update Policy Reactions")
    table_classes = (project_tables.UpdatePolicyReactionTable,)
    template_name = "detail.html"
    page_title = "Scaling Policy {{ scaling_policy.name }}"
    failure_url = URL_PREFIX + "index"

    def __init__(self, *args, **kwargs):
        super(PolicyDetailView, self).__init__(*args, **kwargs)

    # every table needs a get_<tablename>_data method
    def get_updatepolicyreactiontable_data(self):
        try:
            scaling_policy = self._get_data()
            reactions = scaling_policy["reactions"]
        except Exception as ex:
            LOG.exception(ex)
            redirect = self.failure_url
            msg = _("Unable to retrieve scaling policy reactions: %s") % exceptions
            exceptions.handle(self.request, msg, redirect=redirect)

        position = 0
        for reaction in reactions:
            reaction['position'] = position
            position += 1

        return reactions

    def _get_data(self):
        try:
            id = self.kwargs['scaling_policy_id']
            policy = api.get_a10_scaling_policy(self.request, id)
        except Exception:
            msg = _('Unable to retrieve details for scaling policy "%s".') \
                % (id)
            exceptions.handle(self.request, msg,
                              redirect=self.failure_url)
        return policy

    def get_context_data(self, **kwargs):
        context = super(PolicyDetailView, self).get_context_data(**kwargs)
        policy = self._get_data()

        context["scaling_policy"] = policy
        context["url"] = self.get_redirect_url()

        return context

    @staticmethod
    def get_redirect_url():
        return reverse_lazy("horizon:project:a10scaling:index")

    class Meta(object):
        name = "scalingpolicydetail"
        verbose_name = _("Scaling Policy Details")


class AddAlarmView(workflows.WorkflowView):
    name = _("Create Alarm")
    workflow_class = project_workflows.AddScalingAlarmWorkflow
    success_url = reverse_lazy("horizon:project:a10scaling:index")


class UpdateAlarmView(forms.views.ModalFormView):
    name = _("Update Alarm")
    title = name
    form_class = project_forms.UpdateAlarm
    context_object_name = "scaling_alarm"
    success_url = reverse_lazy(URL_PREFIX + "index")
    template_name = "alarm/update.html"
    page_title = "Update Alarm"

    def get_context_data(self, **kwargs):
        context = super(UpdateAlarmView, self).get_context_data(**kwargs)
        return context

    @memoized.memoized_method
    def _get_object(self, *args, **kwargs):
        id = self.kwargs['id']
        self.submit_url = reverse_lazy(URL_PREFIX + "updatealarm",
                                       kwargs={"id": id})
        if id:
            try:
                return api.get_a10_scaling_alarm(self.request, id)
            except Exception as ex:
                msg = _("Unable to retrieve scaling alarm: %s") % ex
                exceptions.handle(self.request, msg, redirect=redirect)

    def get_initial(self):
        rv = self._get_object()
        return rv


class AddActionView(workflows.WorkflowView):
    name = _("Create Action")
    workflow_class = project_workflows.AddActionWorkflow
    success_url = reverse_lazy(URL_PREFIX + "index")


class UpdateActionView(forms.views.ModalFormView):
    name = _("Update Action")
    form_class = project_forms.UpdateAction
    context_object_name = "scaling_action"
    success_url = reverse_lazy(URL_PREFIX + "index")
    template_name = "action/update.html"

    def get_context_data(self, **kwargs):
        context = super(UpdateActionView, self).get_context_data(**kwargs)
        return context

    @memoized.memoized_method
    def _get_object(self, *args, **kwargs):
        id = self.kwargs['id']
        self.submit_url = reverse_lazy(URL_PREFIX + "updateaction",
                                       kwargs={"id": id})
        if id:
            try:
                return api.get_a10_scaling_action(self.request, id)
            except Exception as ex:
                msg = _("Unable to retrieve scaling action: %s") % ex
                exceptions.handle(self.request, msg, redirect=redirect)

    def get_initial(self):
        rv = self._get_object()
        return rv


class AddReactionView(workflows.WorkflowView):
    name = _("Create Reaction")
    workflow_class = project_workflows.AddReactionWorkflow
    success_url = reverse_lazy(URL_PREFIX + "index")
    template_name = "reaction/create.html"

    def get_context_data(self, **kwargs):
        return super(AddReactionView, self).get_context_data(**kwargs)

    def get_initial(self):
        self.submit_url = reverse_lazy(URL_PREFIX + "addreaction",
                                       kwargs=self.kwargs)
        return self.kwargs

    # def post(self, request, *args, **kwargs):
    #     return super(AddReactionView, self).post(request, *args, **kwargs)


class GroupDetailView(tables.MultiTableView):
    name = _("Scaling Group Overview")
    table_classes = (project_tables.A10ScalingGroupMemberTable,)
    template_name = "group/detail.html"
    page_title = "Scaling Group {{ scaling_group.name }}"
    failure_url = URL_PREFIX + "index"

    def __init__(self, *args, **kwargs):
        super(GroupDetailView, self).__init__(*args, **kwargs)

    # every table needs a get_<tablename>_data method
    def get_a10scalinggroupmembertable_data(self):
        try:
            group = self._get_data()
            workers = group.get("workers", [])

        except Exception as ex:
            LOG.exception(ex)
            redirect = self.failure_url
            msg = _("Unable to retrieve scaling group: %s") % exceptions
            exceptions.handle(self.request, msg, redirect=redirect)

        return workers

    def _get_data(self):
        try:
            id = self.kwargs['scaling_group_id']
            group = api.get_a10_scaling_group_with_children(self.request, id)
        except Exception:
            msg = _('Unable to retrieve details for scaling group "%s".') \
                % (id)
            exceptions.handle(self.request, msg,
                              redirect=self.failure_url)
        return group

    def get_context_data(self, **kwargs):
        context = super(GroupDetailView, self).get_context_data(**kwargs)
        group = self._get_data()

        context["scaling_group"] = group
        context["url"] = self.get_redirect_url()

        return context

    @staticmethod
    def get_redirect_url():
        # update redirect to go back to scaling group.
        return reverse_lazy("horizon:project:a10scaling:index")

    class Meta(object):
        name = "scalinggroupdetail"
        verbose_name = _("Scaling Group Details")



# Copyright 2015,  A10 Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from django.conf.urls import patterns
from django.conf.urls import url

from a10_horizon.dashboard.a10networks.a10appliances import views


urlpatterns = patterns(
    'a10_horizon.dashboard.a10networks.a10appliances.views',
    url(r'^$', views.IndexView.as_view(), name='index')
    # url(r'^deleteappliance$', views.DeleteApplianceView.as_view(), name='deleteappliance')
    # url(r'^addimage$', views.AddImageView.as_view(), name="addimage")
)



# Copyright (C) 2014-2016, A10 Networks Inc. All rights reserved.

import logging

from django.utils.translation import ugettext_lazy as _
from horizon import tabs
from horizon import exceptions

from a10_horizon.dashboard.api import scaling as scaling_api
import tables


LOG = logging.getLogger(__name__)


class A10ScalingGroupsTab(tabs.TableTab):
    table_classes = (tables.A10ScalingGroupTable,)
    name = _("Scaling Groups")
    slug = "a10scalinggroups"
    template_name = "horizon/common/_detail_table.html"
    preload = False

    def get_a10scalinggrouptable_data(self):
        try:
            rv = []
            tenant_id = self.request.user.tenant_id

            rv = scaling_api.get_a10_scaling_groups(self.tab_group.request,
                                                    tenant_id=tenant_id)
        except Exception as ex:
            rv = []
            errmsg = "Unable to retrieve scaling group list"
            exceptions.handle(self.tab_group.request, errmsg)
            LOG.exception(ex)

        return rv


class A10ScalingGroupMembersTab(tabs.TableTab):
    table_classes = (tables.A10ScalingGroupMemberTable,)
    name = _("Scaling Group Members")
    slug = "a10scalinggroupmembers"
    template_name = "horizon/common/_detail_table.html"
    preload = False

    def get_a10scalinggroupmemberstable_data(self):
        try:
            tenant_id = self.request.user.tenant_id
            rv = []
            rv = scaling_api.get_a10_scaling_group_members(self.tab_group.request,
                                                           tenant_id=tenant_id)
        except Exception as ex:
            rv = []
            LOG.exception(ex)
            errmsg = _("Unable to retrieve scaling group member list")
            exceptions.handle(self.tab_group.request, errmsg)

        return rv


class A10ScalingPoliciesTab(tabs.TableTab):
    table_classes = (tables.A10ScalingPolicyTable,)
    name = _("Scaling Group Policies")
    slug = "a10scalinggrouppolicies"
    template_name = "horizon/common/_detail_table.html"
    preload = False

    def get_a10scalingpolicytable_data(self):
        try:
            tenant_id = self.request.user.tenant_id
            rv = []
            rv = scaling_api.get_a10_scaling_policies(self.tab_group.request, tenant_id=tenant_id)
        except Exception as ex:
            rv = []
            LOG.exception(ex)
            errmsg = _("Unable to retrieve scaling policy list")
            exceptions.handle(self.tab_group.request, errmsg)

        return rv


class A10ScalingActionTab(tabs.TableTab):
    table_classes = (tables.A10ScalingActionTable,)
    name = _("Scaling Actions")
    slug = "a10scalingactions"
    template_name = "horizon/common/_detail_table.html"
    preload = False

    def get_a10scalingactiontable_data(self):
        try:
            tenant_id = self.request.user.tenant_id
            rv = []
            rv = scaling_api.get_a10_scaling_actions(self.tab_group.request, tenant_id=tenant_id)
        except Exception as ex:
            rv = []
            LOG.exception(ex)
            errmsg = _("Unable to retrieve scaling action list")
            exceptions.handle(self.tab_group.request, errmsg)

        return rv


class A10ScalingAlarmTab(tabs.TableTab):
    table_classes = (tables.A10ScalingAlarmTable,)
    name = _("Scaling Alarms")
    slug = "a10scalingalarms"
    template_name = "horizon/common/_detail_table.html"
    preload = False

    def get_a10scalingalarmtable_data(self):
        try:
            tenant_id = self.request.user.tenant_id
            rv = []
            rv = scaling_api.get_a10_scaling_alarms(self.tab_group.request, tenant_id=tenant_id)
        except Exception as ex:
            rv = []
            LOG.exception(ex)
            errmsg = _("Unable to retrieve scaling alarm list")
            exceptions.handle(self.tab_group.request, errmsg)

        return rv


class A10ScalingTabs(tabs.TabGroup):
    slug = "a10scalingtabs"
    tabs = (A10ScalingGroupsTab,
            A10ScalingPoliciesTab,
            A10ScalingActionTab,
            A10ScalingAlarmTab)
    sticky = True



# Copyright (C) 2016, A10 Networks Inc. All rights reserved.

from django.conf.urls import include
from django.conf.urls import patterns
from django.conf.urls import url

import views

urlpatterns = patterns(

    'a10_horizon.dashboard.a10networks.a10scaling.views',
    url(r'^$', views.IndexView.as_view(), name='index'),
    url(r'^addcertificate/$', views.AddCertificateView.as_view(),
        name='addcertificate'),
    url(r'^updatecertificate/(?P<certificate_id>[^/]+)/',
        views.UpdateCertificateView.as_view(), name='updatecertificate'),
    url(r'^addcertificatebinding/$', views.AddCertificateBindingView.as_view(),
        name="addcertificatebinding"),
)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from django.utils.translation import ugettext_lazy as _
import logging

import horizon

LOG = logging.getLogger(__name__)


class A10SSLPanel(horizon.Panel):
    name = _("SSL")
    slug = "a10ssl"
    permissions = ("openstack.services.network", )

    # def can_register(self):
    #     rv = False

    #     try:
    #         import a10_openstack  # noqa
    #         rv = True
    #     except Exception:
    #         msg = "{0} cannot be displayed.  Please contact A10 Account Team to enable."
    #         rv = False
    #         LOG.exception(msg.format("A10 SSL"))
    #     return rv



# Copyright (C) 2014-2015, A10 Networks Inc. All rights reserved.

import logging

from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import workflows

# a10_horizon.dashboard.api.client.Client extends neutron.api.client.Client
from a10_horizon.dashboard.api import certificates as api
from openstack_dashboard.api import lbaas as lbaas_api

# TODO(Pull these from A10 constants)
AVAILABLE_PROTOCOLS = ('HTTP', 'HTTPS', 'TCP')
AVAILABLE_METHODS = ('ROUND_ROBIN', 'LEAST_CONNECTIONS', 'SOURCE_IP')

LOG = logging.getLogger(__name__)


class AddCertificateAction(workflows.Action):
    name = forms.CharField(label=_("Name"), min_length=1, max_length=255,
                           required=True)
    description = forms.CharField(label=_("Description"), min_length=1,
                                  max_length=255, required=False)
    cert_data = forms.CharField(
        label=_("Certificate Data"), min_length=1, max_length=8000,
        widget=forms.Textarea(attrs={'cols': 80, 'rows': 15}), required=True)
    key_data = forms.CharField(
        label=_("Key Data"), min_length=1, max_length=8000,
        widget=forms.Textarea(attrs={'cols': 80, 'rows': 15}), required=True)
    password = forms.CharField(
        label=_("Password"), min_length=1, max_length=1024,
        widget=forms.PasswordInput(render_value=False), required=False)
    intermediate_data = forms.CharField(
        label=_("Intermediate Certificate Data"), min_length=1, max_length=8000,
        widget=forms.Textarea(attrs={'cols': 80, 'rows': 15}), required=False)

    def clean(self):
        cleaned_data = super(AddCertificateAction, self).clean()
        LOG.debug("AddCertificateAction:clean(): cleaned_data=%s" % cleaned_data)
        file = cleaned_data.get('cert_file')
        data = cleaned_data.get('cert_data')
        LOG.info("file: {}".format(file))
        LOG.info("data: {}".format(data))
        LOG.info("self.files: {}".format(self.files))
        LOG.info("self.request.FILES: {}".format(self.request.FILES.keys()))
        if file and data:
            msg = _("You may only enter certificate data if you are not uploading the certificate.")
            self._errors['cert_data'] = self.error_class([msg])
        return cleaned_data

    class Meta(object):
        name = _("Add New Certificate")
        permissions = ('openstack.services.network',)
        help_text = _("Add a certificate.\n\n Something about using a file or "
                      "copy-pasta.")  # FIXME(traff)


class AddCertificateStep(workflows.Step):
    action_class = AddCertificateAction
    contributes = ("name", "description", "cert_data",
                   "key_data", "intermediate_data", "password")


class AddCertificate(workflows.Workflow):
    slug = "addcertificate"
    name = _("Add Certificate")
    finalize_button_name = _("Add")
    success_message = _('Added certificate')
    failure_message = _('Unable to add certificate')
    success_url = "horizon:project:a10ssl:index"
    default_steps = (AddCertificateStep,)

    def handle(self, request, context):
        try:
            LOG.debug("AddCertificate:handle(): context=%s" % (context))
            context['certificate_id'] = api.certificate_create(
                request, **context).get('id')
            return True
        except Exception as ex:
            LOG.error("Could not add certificate. ERROR=%s" % ex)
            exceptions.handle(request, _("Unable to add certificate."))
        return False


class AddCertificateBindingAction(workflows.Action):
    vip_id = forms.ChoiceField(label=_("VIP"))
    certificate_id = forms.ChoiceField(label=_("Certificate"))

    def __init__(self, request, *args, **kwargs):
        certificates = []
        vips = []
        super(AddCertificateBindingAction, self).__init__(request, *args, **kwargs)

        tenant_id = request.user.tenant_id

        vip_id_choices = [('', _("Select a VIP"))]
        certificate_id_choices = [('', _("Select a Certificate"))]

        try:
            kwargs["tenant_id"] = tenant_id
            vips = lbaas_api.vip_list(request, **kwargs)
        except Exception as ex:
            LOG.error("Could not add certificate. ERROR=%s" % ex)
            exceptions.handle(request, _("Unable to retrieve VIPs list"))

        try:
            certificates = api.certificate_list(request, **kwargs)
        # TODO(mdurrant) narrow exception handling
        except Exception as ex:
            LOG.error("Could retrieve certificate list. ERROR=%s" % ex)
            exceptions.handle(request, _("Unable to retrieve Certificates"))

        for v in vips:
            # we only care about HTTPS connections
            if (v.protocol == "HTTPS"):
                vip_id_choices.append((v.id, v.name))

        for c in certificates:
            certificate_id_choices.append((c.id, c.name))

        self.fields['vip_id'].choices = vip_id_choices
        self.fields['certificate_id'].choices = certificate_id_choices

    def clean(self):
        cleaned_data = super(AddCertificateBindingAction, self).clean()
        # TODO(mdurrant): Not sure there's any sanitation to be performed here
        return cleaned_data

    class Meta(object):
        name = _("Add New Certificate Association")
        # FIXME(mdurrant) - API policy covers these but we should probably double-check
        permissions = ('openstack.services.network',)
        help_text = _("Assign a Certificate to VIP\n\n")


class AddCertificateBindingStep(workflows.Step):
    action_class = AddCertificateBindingAction
    contributes = ("vip_id", "certificate_id")


class AddCertificateBinding(workflows.Workflow):
    slug = "addcertificatebinding"
    name = _("Add VIP Association")
    finalize_button_name = ("Create Association")
    success_message = _("Added association between VIP:'%s' and Certificate:'%s'.")  # noqa
    failure_message = _("Unable to create association for VIP:'%s',Certificate:'%s'")  # noqa
    default_steps = (AddCertificateBindingStep,)
    success_url = "horizon:project:a10ssl:index"

    def format_status_message(self, message):
        vip_id = self.context.get('vip_id')
        certificate_id = self.context.get('certificate_id')
        return message % (vip_id, certificate_id)

    def handle(self, request, context):
        # make sure they both exist.
        try:
            api.certificate_binding_create(request, **context)
            return True
        # TODO(mdurrant) narrow exception handling
        except Exception as ex:
            LOG.error("Could not add certificate association. ERROR=%s" % ex)
            return False



# Copyright (C) 2014-2015, A10 Networks Inc. All rights reserved.
from django.conf.urls import url
from django.conf.urls import include
from django.conf.urls import patterns
from django.conf import settings
from django.conf.urls import static

import a10_horizon
from a10devices import urls as device_urls
from a10scaling import urls as scaling_urls
from a10ssl import urls as ssl_urls
import views


urlpatterns = patterns("",
    url(r'^a10networks/', views.IndexView.as_view(), name='index'),
    url(r'^a10ssl/', include(ssl_urls, app_namesppace="a10networks", namespace="a10ssl")),
    url(r'^a10scaling/', include(scaling_urls, app_namesppace="a10networks", namespace="a10scaling")),
    url(r'^a10devices/', include(device_urls, app_namesppace="a10networks", namespace="a10devices"))
)



# Copyright (C) 2014-2015, A10 Networks Inc. All rights reserved.

from django.core.urlresolvers import reverse_lazy
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon.utils import memoized
from horizon import messages
from horizon import tabs
from horizon import workflows
import logging

from a10_horizon.dashboard.api import certificates as cert_api
import forms as project_forms
import workflows as project_workflows
import tabs as project_tabs

import re

LOG = logging.getLogger(__name__)


ACTION = "action"
NOUN = "noun"
PLURAL = "plural"


class IndexView(tabs.TabView):
    tab_group_class = (project_tabs.A10SSLTabs)
    template_name = 'ssl_tabs.html'

    delete_actions = {
        "certificate": {
            ACTION: cert_api.certificate_delete,
            NOUN: "Certificate",
            PLURAL: "Certificates"
        },
        "certificatebinding": {
            ACTION: cert_api.certificate_binding_delete,
            NOUN: "Certificate Association",
            PLURAL: "Certificate Associations"
        },
    }

    def post(self, request, *args, **kwargs):
        obj_ids = request.POST.getlist('object_ids')
        action = request.POST['action']
        m = re.search('.delete([a-z]+)', action).group(1)
        if obj_ids == []:
            obj_ids.append(re.search('([0-9a-z-]+)$', action).group(1))

        if m in self.delete_actions:
            delete_action = self.delete_actions[m]
            for obj_id in obj_ids:
                success_msg = "Deleted {0} {1}".format(delete_action[NOUN], obj_id)
                failure_msg = "Unable to delete {0} {1}".format(delete_action[NOUN], obj_id)

                try:
                    delete_action[ACTION](request, obj_id)
                    messages.success(request, success_msg)
                except Exception as ex:
                    exceptions.handle(request, failure_msg)
                    LOG.exception(ex)

        return self.get(request, *args, **kwargs)


class AddCertificateView(workflows.WorkflowView):
    workflow_class = project_workflows.AddCertificate


class AddCertificateBindingView(workflows.WorkflowView):
    workflow_class = project_workflows.AddCertificateBinding


class UpdateCertificateView(forms.ModalFormView):
    form_class = project_forms.UpdateCertificate
    template_name = "certificates/updatecertificate.html"
    context_object_name = 'certificate'
    success_url = reverse_lazy("horizon:project:a10ssl:index")

    def get_context_data(self, **kwargs):
        context = super(UpdateCertificateView, self).get_context_data(**kwargs)
        context["certificate_id"] = self.kwargs['certificate_id']
        return context

    @memoized.memoized_method
    def _get_object(self, *args, **kwargs):
        certificate_id = self.kwargs['certificate_id']
        try:
            return cert_api.certificate_get(self.request, certificate_id)
        except Exception as e:
            redirect = self.success_url
            msg = _('Unable to retrieve certificate. %s') % e
            exceptions.handle(self.request, msg, redirect=redirect)

    def get_initial(self):
        certificate = self._get_object()  # FIXME(traff)
        # return {'certificate_id': certificate['id'], 'name': 'hello_world',
        return certificate



# Copyright (C) 2015 A10 Networks Inc. All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from __future__ import absolute_import

import logging
from openstack_dashboard.api.neutron import NeutronAPIDictWrapper
from openstack_dashboard.api.neutron import neutronclient

from a10_neutronclient.resources import a10_device_instance

LOG = logging.getLogger(__name__)


class A10Appliance(NeutronAPIDictWrapper):
    """Wrapper for a10_appliance dictionary"""
    def __init__(self, apiresource):
        super(A10Appliance, self).__init__(apiresource)


def get_a10_appliances(request, **kwargs):
    rv = neutronclient(request).list_a10_device_instances(**kwargs).get(a10_device_instance.RESOURCES)
    return map(A10Appliance, rv)


def get_a10_appliance(request, id, **params):
    rv = neutronclient(request).show_a10_device_instance(id).get(a10_device_instance.RESOURCE)
    return A10Appliance(rv)


def delete_a10_appliance(request, id):
    neutronclient(request).delete_a10_device_instances(id)


def create_a10_appliance(request, **kwargs):
    body = {a10_device_instance.RESOURCE: kwargs}
    rv = neutronclient(request).create_a10_device_instances(body=body).get(a10_device_instance.RESOURCE)
    return A10Appliance(rv)


def update_a10_appliance(request, id, **kwargs):
    body = {a10_device_instance.RESOURCE: kwargs}
    rv = neutronclient(request).update_a10_device_instances(id, body=body).get(a10_device_instance.RESOURCE)
    return A10Appliance(rv)



# Copyright (C) 2016, A10 Networks Inc. All rights reserved.
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


# The slug of the panel to be added to HORIZON_CONFIG. Required.
PANEL = 'a10scaling'
# The slug of the dashboard the PANEL associated with. Required.
PANEL_DASHBOARD = 'project'
# The slug of the panel group the PANEL is associated with.
PANEL_GROUP = 'a10networks'

# Python panel class of the PANEL to be added.
ADD_PANEL = 'a10_horizon.dashboard.a10networks.a10scaling.panel.A10ScalingPanel'
ADD_INSTALLED_APPS = ['a10_horizon.dashboard.a10networks.a10scaling']
AUTO_DISCOVER_STATIC_FILES = True



# Copyright (C) 2015 A10 Networks Inc. All rights reserved.

from __future__ import absolute_import

import logging

from django.conf import settings

from horizon.utils.memoized import memoized  # noqa

from openstack_dashboard.api import base

from openstack_dashboard.api.neutron import NeutronAPIDictWrapper
# a10 client that extends neutronclient.v2_0.client.Client
from a10_openstack.neutron_ext.api import client as neutron_client

LOG = logging.getLogger(__name__)


class Certificate(NeutronAPIDictWrapper):
    """Wrapper for neutron Certificates"""
    def __init__(self, apiresource):
        super(Certificate, self).__init__(apiresource)


class CertificateBinding(NeutronAPIDictWrapper):
    """Wrapper for neutron CertificateBindings"""
    def __init__(self, apiresource):
        super(CertificateBinding, self).__init__(apiresource)


def neutronclient(request):
    insecure = getattr(settings, 'OPENSTACK_SSL_NO_VERIFY', False)
    cacert = getattr(settings, 'OPENSTACK_SSL_CACERT', None)
    LOG.debug('neutronclient connection created using token "%s" and url "%s"'
              % (request.user.token.id, base.url_for(request, 'network')))
    LOG.debug('user_id=%(user)s, tenant_id=%(tenant)s' %
              {'user': request.user.id, 'tenant': request.user.tenant_id})
    c = neutron_client.Client(token=request.user.token.id,
                              auth_url=base.url_for(request, 'identity'),
                              endpoint_url=base.url_for(request, 'network'),
                              insecure=insecure,
                              ca_cert=cacert)
    return c


def certificate_list(request, **params):
    LOG.debug("certificates_list(): params=%s" % (params))
    certificates = []
    certificates = neutronclient(request).list_certificates(**params).get('certificates')
    return map(Certificate, certificates)


def certificate_get(request, certificate_id, **params):
    # TODO(mdurrant): Add option to get bindings w/ cert.
    LOG.debug("certificate_get(): certificate_id=%s, params=%s" % (certificate_id, params))
    certificate = neutronclient(request).show_certificate(certificate_id,
                                                          **params).get('certificate')
    return Certificate(certificate)


def certificate_create(request, **kwargs):
    """Create specified Certificate"""
    body = {"certificate": kwargs}
    LOG.debug("certificate_create(): kwargs=%s,body=%s" % (kwargs, body))
    certificate = neutronclient(request).create_certificate(body=body).get('certificate')
    return Certificate(certificate)


def certificate_update(request, **kwargs):
    body = {"certificate": kwargs}
    LOG.debug("certificate_update(): kwargs=%s", (kwargs))
    certificate = neutronclient(request).update_certificate(body=body).get('certificate')
    return Certificate(certificate)


def certificate_delete(request, certificate_id):
    LOG.debug("certificate_delete(): certificiate_id:%s" % certificate_id)
    # TODO(mmd): Should this return status or do we assume it always works?
    neutronclient(request).delete_certificate(certificate_id)


def certificate_bindings_list(request, **params):
    LOG.debug("certificate_bindings_list(): params={}".format(params))

    bindings = \
        neutronclient(request).list_certificate_bindings(**params).get('certificate_bindings')
    return map(CertificateBinding, bindings)


def certificate_binding_get(request, binding_id, **params):
    LOG.debug("certificate_binding_get(): binding_id=%s, params=%s" % (binding_id, params))
    binding = neutronclient(request).show_certificate_binding(binding_id,
                                                              **params).get('certificate_binding')
    return CertificateBinding(binding)


def certificate_binding_create(request, **kwargs):
    """Binding specified Certificate ID to specified VIP ID"""
    LOG.debug("certificate_binding_create(): request=%s, kwargs=%s" % (request, kwargs))
    body = {'certificate_binding': kwargs}

    binding = \
        neutronclient(request).create_certificate_binding(body=body).get('certificate_binding')
    return CertificateBinding(binding)


def certificate_binding_delete(request, binding_id):
    LOG.debug("certificate_binding_delete(): binding_id=%s" % binding_id)
    neutronclient(request).delete_certificate_binding(binding_id)



# Copyright (C) 2014-2015, A10 Networks Inc. All rights reserved.

import importlib
import logging
import re

from django.core.urlresolvers import reverse
from django.core.urlresolvers import resolve
from django.utils.translation import ugettext_lazy as _


logger = logging.getLogger(__name__)


def post_hack(self, request, *args, **kwargs):
    # These import statements break the pip install when they're global
    from a10_horizon.dashboard.api import certificates as cert_api
    from horizon import exceptions
    from horizon import messages
    from openstack_dashboard import api

    obj_ids = request.POST.getlist('object_ids')
    action = request.POST['action']
    m = re.search('.delete([a-z]+)', action).group(1)
    if obj_ids == []:
        obj_ids.append(re.search('([0-9a-z-]+)$', action).group(1))
    if m == 'monitor':
        for obj_id in obj_ids:
            try:
                api.lbaas.pool_health_monitor_delete(request, obj_id)
                messages.success(request, _('Deleted monitor %s') % obj_id)
            except Exception as e:
                exceptions.handle(request,
                                  _('Unable to delete monitor. %s') % e)
    if m == 'pool':
        for obj_id in obj_ids:
            try:
                api.lbaas.pool_delete(request, obj_id)
                messages.success(request, _('Deleted pool %s') % obj_id)
            except Exception as e:
                exceptions.handle(request,
                                  _('Unable to delete pool. %s') % e)
    if m == 'member':
        for obj_id in obj_ids:
            try:
                api.lbaas.member_delete(request, obj_id)
                messages.success(request, _('Deleted member %s') % obj_id)
            except Exception as e:
                exceptions.handle(request,
                                  _('Unable to delete member. %s') % e)
    if m == 'vip':
        for obj_id in obj_ids:
            try:
                vip_id = api.lbaas.pool_get(request, obj_id).vip_id
            except Exception as e:
                exceptions.handle(request,
                                  _('Unable to locate VIP to delete. %s')
                                  % e)
            if vip_id is not None:
                try:
                    api.lbaas.vip_delete(request, vip_id)
                    messages.success(request, _('Deleted VIP %s') % vip_id)
                except Exception as e:
                    exceptions.handle(request,
                                      _('Unable to delete VIP. %s') % e)

    if m == 'certificate':
        for obj_id in obj_ids:
            try:
                cert_api.certificate_delete(request, obj_id)
                messages.success(request, _("Deleted certificate. %s") % obj_id)
            except Exception as e:
                exceptions.handle(request, _('Unable to delete Certificate. %s') % e)

    if m == 'certificatebinding':
        for obj_id in obj_ids:
            try:
                cert_api.certificate_binding_delete(request, obj_id)
                messages.success(request, _("Deleted certificate association. %s") % obj_id)
            except Exception as e:
                exceptions.handle(request, _("Unable to delete certificate association %s") %
                                  obj_id)

    return self.get(request, *args, **kwargs)


def add_tab(url, tab, idx=None):
    logger.debug("add_tab(), url:%s, tabl: %s, idx: %s", url, tab, idx)
    # Get the index view via it's url name
    view_func = resolve(reverse(url)).func
    module = importlib.import_module(view_func.__module__)
    view = getattr(module, view_func.__name__)

    # Import the view and if it has a tab_group_class
    # we'll add to it.
    if hasattr(view, 'tab_group_class'):
        tab_list = list(view.tab_group_class.tabs)
        if tab not in tab_list:
            if idx is None:
                idx = len(tab_list)

            tab_list.insert(idx, tab)
            view.tab_group_class.tabs = tuple(tab_list)



# Copyright (C) 2016 A10 Networks Inc. All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

import logging
from openstack_dashboard.api import neutron

from a10_neutronclient.resources import a10_scaling_group

neutronclient = neutron.neutronclient
NeutronAPIDictWrapper = neutron.NeutronAPIDictWrapper


LOG = logging.getLogger(__name__)


class A10ScalingGroup(NeutronAPIDictWrapper):
    """Wrapper for a10_scaling_group dictionary"""
    def __init__(self, apiresource):
        super(A10ScalingGroup, self).__init__(apiresource)


class A10ScalingGroupMember(NeutronAPIDictWrapper):
    """Wrapper for a10_scaling_group_member dictionary"""
    def __init__(self, apiresource):
        super(A10ScalingGroupMember, self).__init__(apiresource)


class A10ScalingPolicy(NeutronAPIDictWrapper):
    """Wrapper for a10_scaling_group dictionary"""
    def __init__(self, apiresource):
        super(A10ScalingPolicy, self).__init__(apiresource)


class A10ScalingPolicyReaction(NeutronAPIDictWrapper):
    """Wrapper for a10_scaling_group dictionary"""
    def __init__(self, apiresource):
        super(A10ScalingPolicyReaction, self).__init__(apiresource)


class A10ScalingAction(NeutronAPIDictWrapper):
    """Wrapper for a10_scaling_group dictionary"""
    def __init__(self, apiresource):
        super(A10ScalingAction, self).__init__(apiresource)


class A10ScalingAlarm(NeutronAPIDictWrapper):
    """Wrapper for a10_scaling_group dictionary"""
    def __init__(self, apiresource):
        super(A10ScalingAlarm, self).__init__(apiresource)


# Scaling Groups

def get_a10_scaling_groups(request, **kwargs):
    rv = neutronclient(request)\
        .list_a10_scaling_groups(**kwargs)\
        .get(a10_scaling_group.SCALING_GROUPS)
    return map(A10ScalingGroup, rv)


def get_a10_scaling_group(request, id, **params):
    rv = neutronclient(request).show_a10_scaling_group(id).get(a10_scaling_group.SCALING_GROUP)
    return A10ScalingGroup(rv)


def get_a10_scaling_group_with_children(request, id, **params):
    group = neutronclient(request).show_a10_scaling_group(id).get(a10_scaling_group.SCALING_GROUP)
    worker_filter = {
        "scaling_group_id": id
    }

    workers = neutronclient(request)\
        .list_a10_scaling_group_workers(filters=worker_filter)\
        .get(a10_scaling_group.SCALING_GROUP_WORKERS)
    group["workers"] = map(A10ScalingGroupMember, workers)
    # group["workers"] = workers
    rv = A10ScalingGroup(group)
    return rv


def delete_a10_scaling_group(request, id):
    neutronclient(request).delete_a10_scaling_group(id)


def create_a10_scaling_group(request, **kwargs):
    body = {a10_scaling_group.SCALING_GROUP: kwargs}
    rv = neutronclient(request)\
        .create_a10_scaling_group(body=body)\
        .get(a10_scaling_group.SCALING_GROUP)
    return A10ScalingGroup(rv)


def update_a10_scaling_group(request, id, **kwargs):
    body = {a10_scaling_group.SCALING_GROUP: kwargs}
    rv = neutronclient(request)\
        .update_a10_scaling_group(id, body=body)\
        .get(a10_scaling_group.SCALING_GROUP)
    return A10ScalingGroup(rv)


# Scaling Policy

def get_a10_scaling_policies(request, **kwargs):
    rv = neutronclient(request)\
        .list_a10_scaling_policies(**kwargs)\
        .get(a10_scaling_group.SCALING_POLICIES)
    return map(A10ScalingPolicy, rv)


def get_a10_scaling_policy(request, id, **params):
    rv = neutronclient(request).show_a10_scaling_policy(id).get(a10_scaling_group.SCALING_POLICY)
    return A10ScalingPolicy(rv)


def delete_a10_scaling_policy(request, id):
    neutronclient(request).delete_a10_scaling_policy(id)


def create_a10_scaling_policy(request, **kwargs):
    body = {a10_scaling_group.SCALING_POLICY: kwargs}
    rv = neutronclient(request)\
        .create_a10_scaling_policy(body=body)\
        .get(a10_scaling_group.SCALING_POLICY)
    return A10ScalingPolicy(rv)


def update_a10_scaling_policy(request, id, **kwargs):
    body = {a10_scaling_group.SCALING_POLICY: kwargs}
    rv = neutronclient(request)\
        .update_a10_scaling_policy(id, body=body)\
        .get(a10_scaling_group.SCALING_POLICY)
    return A10ScalingPolicy(rv)


# Scaling Alarms

def get_a10_scaling_alarms(request, **kwargs):
    rv = neutronclient(request)\
        .list_a10_scaling_alarms(**kwargs)\
        .get(a10_scaling_group.SCALING_ALARMS)
    return map(A10ScalingAlarm, rv)


def get_a10_scaling_alarm(request, id, **kwargs):
    rv = neutronclient(request)\
        .show_a10_scaling_alarm(id)\
        .get(a10_scaling_group.SCALING_ALARM)
    return A10ScalingAlarm(rv)


def delete_a10_scaling_alarm(request, id):
    neutronclient(request).delete_a10_scaling_alarm(id)


def create_a10_scaling_alarm(request, **kwargs):
    body = {a10_scaling_group.SCALING_ALARM: kwargs}
    rv = neutronclient(request)\
        .create_a10_scaling_alarm(body=body)\
        .get(a10_scaling_group.SCALING_ALARM)
    return A10ScalingAlarm(rv)


def update_a10_scaling_alarm(request, id, **kwargs):
    body = {a10_scaling_group.SCALING_ALARM: kwargs}
    rv = neutronclient(request)\
        .update_a10_scaling_alarm(id, body=body).get(a10_scaling_group.SCALING_ALARM)
    return A10ScalingAlarm(rv)


# Scaling Actions

def get_a10_scaling_actions(request, **kwargs):
    rv = neutronclient(request)\
        .list_a10_scaling_actions(**kwargs)\
        .get(a10_scaling_group.SCALING_ACTIONS)
    return map(A10ScalingAction, rv)


def get_a10_scaling_action(request, id, **kwargs):
    rv = neutronclient(request).show_a10_scaling_action(id).get(a10_scaling_group.SCALING_ACTION)
    return A10ScalingAction(rv)


def delete_a10_scaling_action(request, id):
    neutronclient(request).delete_a10_scaling_action(id)


def create_a10_scaling_action(request, **kwargs):
    body = {a10_scaling_group.SCALING_ACTION: kwargs}
    rv = neutronclient(request)\
        .create_a10_scaling_action(body=body)\
        .get(a10_scaling_group.SCALING_ACTION)
    return A10ScalingAction(rv)


def update_a10_scaling_action(request, id, **kwargs):
    body = {a10_scaling_group.SCALING_ACTION: kwargs}
    rv = neutronclient(request)\
        .update_a10_scaling_action(id, body=body)\
        .get(a10_scaling_group.SCALING_ACTION)
    return A10ScalingAction(rv)


# # Scaling Members
# def get_a10_scaling_members(request, id, **kwargs):
#     return A10ScalingMember({})



# Copyright (C) 2016, A10 Networks Inc. All rights reserved.

from django.core.urlresolvers import reverse_lazy
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon.utils import memoized
from horizon import messages
from horizon import tabs
from horizon import views
from horizon import workflows

import logging


import re

LOG = logging.getLogger(__name__)


ACTION = "action"
NOUN = "noun"
PLURAL = "plural"


class IndexView(views.HorizonTemplateView):
    pass



# Copyright (C) 2016, A10 Networks Inc. All rights reserved.

import logging

from django.core.urlresolvers import reverse
from django.core.urlresolvers import reverse_lazy
from django.shortcuts import redirect
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ungettext_lazy
from horizon import exceptions
from horizon import messages
from horizon import tables

from constants import AGG_DICT
from constants import OP_DICT
from constants import UNIT_DICT

from a10_horizon.dashboard.api import scaling as scaling_api


LOG = logging.getLogger(__name__)
URL_PREFIX = "horizon:project:a10scaling:"

# This file has a lot of DRY violations sacrificing elegance for ease of maintenance


class AddScalingPolicyLink(tables.LinkAction):
    name = "addscalingpolicy"
    verbose_name = _("Add Scaling Policy")
    url = URL_PREFIX + "addscalingpolicy"
    classes = ("ajax-modal",)
    icon = "plus"
    policy_rules = ("network",)  # FIXME(mdurrant) - A10-specific policies?
    success_url = "horizon:project:a10scaling:index"


class UpdateScalingPolicyLink(tables.LinkAction):
    name = "a10scaling:updatescalingpolicy"
    verbose_name = _("Edit Scaling Policy")
    classes = ("btn-update", "ajax-modal")
    icon = "pencil"
    # url = URL_PREFIX + "updatescalingpolicy"
    # success_url = "horizon:project:a10scaling:index"

    def get_link_url(self, datum):
        base_url = reverse(URL_PREFIX + "updatescalingpolicy",
                           kwargs={'scaling_policy_id': datum["id"]})
        return base_url


class DeleteScalingPolicyAction(tables.DeleteAction):
    name = "deletescalingpolicy"
    verbose_name = _("Delete Scaling Policy")

    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Delete Scaling Policy",
            u"Delete Scaling Policies",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Scheduled deletion of Scaling Policy",
            u"Scheduled deletion of Scaling Policies",
            count
        )

    def handle(self, data_table, request, object_ids):
        for obj_id in object_ids:
            try:
                scaling_api.delete_a10_scaling_policy(request, obj_id)
                redirect = reverse(self.redirect_url)
            except Exception as ex:
                msg = _("Failed to delete scaling policy")
                LOG.exception(ex)
                exceptions.handle(request, msg, redirect=self.redirect_url)

        return redirect(URL_PREFIX + "index")

    def allowed(self, request, obj):
        return True


class AddAlarmLink(tables.LinkAction):
    name = "addscalingalarm"
    verbose_name = _("Add Scaling Alarm")
    url = URL_PREFIX + "addalarm"
    classes = ("ajax-modal",)
    icon = "plus"
    alarm_rules = ("network",)  # FIXME(mdurrant) - A10-specific policies?
    success_url = "horizon:project:a10scaling:index"


class UpdateAlarmLink(tables.LinkAction):
    name = "a10scaling:updatescalingalarm"
    verbose_name = _("Edit Scaling Alarm")
    classes = ("btn-update", "ajax-modal")
    icon = "pencil"
    # url = URL_PREFIX + "updatescalingalarm"
    # success_url = "horizon:project:a10scaling:index"

    def get_link_url(self, datum):
        base_url = reverse(URL_PREFIX + "updatealarm",
                           kwargs={'id': datum["id"]})
        return base_url


class DeleteAlarmLink(tables.DeleteAction):
    name = "deletescalingalarm"
    verbose_name = _("Delete Scaling Alarm")
    redirect_url = reverse_lazy(URL_PREFIX + "index")
    failure_message = _('Failed to delete reaction')
    # method = "GET"

    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Delete Scaling Alarm",
            u"Delete Scaling Alarms",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Scheduled deletion of Scaling Alarm",
            u"Scheduled deletion of Scaling Alarms",
            count
        )

    def handle(self, data_table, request, object_ids):
        for obj_id in object_ids:
            try:
                scaling_api.delete_a10_scaling_alarm(request, obj_id)
                redirect = reverse(self.redirect_url)
            except Exception as ex:
                msg = self.failure_message
                LOG.exception(ex)
                exceptions.handle(request, msg, redirect=self.redirect_url)

            self.success_url = self.redirect_url
            msg = _('Scaling Action deleted.')
            LOG.debug(msg)
            messages.success(request, msg)

        return redirect(self.redirect_url)

    def allowed(self, request, obj):
        return True


class AddActionLink(tables.LinkAction):
    name = "addscalingaction"
    verbose_name = _("Add Scaling Action")
    url = URL_PREFIX + "addaction"
    classes = ("ajax-modal",)
    icon = "plus"
    action_rules = ("network",)  # FIXME(mdurrant) - A10-specific policies?
    success_url = "horizon:project:a10scaling:index"


class UpdateActionLink(tables.LinkAction):
    name = "updatescalingaction"
    verbose_name = _("Edit Scaling Action")
    classes = ("btn-update", "ajax-modal")
    icon = "pencil"

    def get_link_url(self, datum):
        base_url = reverse(URL_PREFIX + "updateaction",
                           kwargs={'id': datum["id"]})
        return base_url


class DeleteActionLink(tables.DeleteAction):
    name = "deletescalingaction"
    verbose_name = _("Delete Scaling Action")
    redirect_url = reverse_lazy(URL_PREFIX + "index")
    failure_message = _('Failed to delete reaction')

    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Delete Scaling Action",
            u"Delete Scaling Actions",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Scheduled deletion of Scaling Action",
            u"Scheduled deletion of Scaling Actions",
            count
        )

    def handle(self, data_table, request, object_ids):
        for obj_id in object_ids:
            try:
                scaling_api.delete_a10_scaling_action(request, obj_id)
                redirect = reverse(self.redirect_url)
            except Exception as ex:
                msg = self.failure_message
                LOG.exception(ex)
                exceptions.handle(request, msg, redirect=self.redirect_url)

            self.success_url = self.redirect_url
            msg = _('Scaling Action deleted.')
            messages.success(request, msg)

        return redirect(self.redirect_url)

    def allowed(self, request, obj):
        return True


class AddReactionLink(tables.LinkAction):
    name = "addreaction"
    verbose_name = _("Add Scaling Reaction")
    classes = ("ajax-modal",)
    icon = "plus"
    url = URL_PREFIX + "addreaction"

    def get_link_url(self, datum=None):
        id = None
        if datum is not None and "id" in datum:
            id = datum["id"]
        else:
            id = self.table.kwargs["scaling_policy_id"]
        return reverse(self.url, kwargs={'scaling_policy_id': id})


class DeleteReactionLink(tables.DeleteAction):
    name = "deletereaction"
    verbose_name = _("Delete Reaction")
    icon = "minus"
    _redirect_url = URL_PREFIX + "scalingpolicydetail"
    failure_message = _('Failed to delete reaction')

    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Delete Reaction",
            u"Delete Reactions",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Scheduled deletion of scaling reaction",
            u"Scheduled deletion of scaling reactions",
            count
        )

    def _remove_deleted(self, reactions, object_ids):
        rv = reactions
        if len(reactions) > 0:
                position = 0
                for reaction in reactions:
                    reaction["position"] = int(position)
                    position += 1

                for obj_id in object_ids:
                    reaction = None
                    try:
                        reaction = reactions[int(obj_id)]
                    except Exception as ex:
                        LOG.exception(ex)

                    if reaction:
                        del reactions[int(obj_id)]

        return rv

    def handle(self, data_table, request, obj_ids):
        scaling_policy_id = data_table.kwargs.get("scaling_policy_id", None)
        self.redirect_url = reverse(self._redirect_url,
                                    kwargs={"scaling_policy_id": scaling_policy_id})
        self.success_url = self.redirect_url
        reaction = None

        try:
            existing_policy = scaling_api.get_a10_scaling_policy(request, scaling_policy_id)
            reactions = self._remove_deleted(existing_policy.reactions, obj_ids)

            if len(reactions) > 0:
                position = 0
                for reaction in reactions:
                    reaction["position"] = int(position)
                    position += 1

                reaction = None
                for obj_id in obj_ids:
                    try:
                        reaction = reactions[int(obj_id)]
                    except Exception as ex:
                        pass
                    if reaction:
                        del reactions[int(obj_id)]

        except Exception as ex:
            msg = self.failure_message
            LOG.exception(ex)
            # redirect = self.redirect_url
            exceptions.handle(request, msg, redirect=redirect)

        existing_policy = existing_policy.to_dict()
        existing_policy["reactions"] = map(lambda x:
                                           {
                                               "alarm_id": x["alarm_id"],
                                               "action_id": x["action_id"]
                                           },
                                           reactions)
        del existing_policy["tenant_id"]

        try:
            scaling_api.update_a10_scaling_policy(request, **existing_policy)
            self.success_url = self.redirect_url
            # redirect = self.success_url
            msg = _('Reaction deleted.')
            LOG.debug(msg)
            messages.success(request, msg)

        except Exception as ex:
            msg = self.failure_message
            LOG.exception(ex)
            # redirect = self.redirect_url
            exceptions.handle(request, msg, redirect=self.redirect_url)

        return redirect(str(self.success_url))

    def allowed(self, request, obj):
        return True


def get_group_detail_link(datum):
    return reverse_lazy(URL_PREFIX + "scalinggroupdetail", kwargs={"scaling_group_id": datum["id"]})


class A10ScalingGroupTable(tables.DataTable):
    # form fields
    id = tables.Column("id", verbose_name=_("ID"), link=get_group_detail_link)
    # TODO(tenant name lookup?)
    tenant_id = tables.Column("tenant_id", verbose_name=_("Tenant ID"), hidden=True)
    name = tables.Column("name", verbose_name="Name", link=get_group_detail_link)
    description = tables.Column("description", verbose_name="Description")

    class Meta(object):
        name = "a10scalinggrouptable"
        verbose_name = _("Scaling Groups")
        table_actions = ()
        row_actions = ()


class A10ScalingGroupMemberTable(tables.DataTable):
    id = tables.Column("id", verbose_name=_("ID"), hidden=True)
    scaling_group_id = tables.Column("scaling_group_id", verbose_name=_("Scaling Group ID"),
                                     hidden=True)
    tenant_id = tables.Column("tenant_id", verbose_name=_("Tenant ID"), hidden=True)
    name = tables.Column("name", verbose_name=("Name"))
    host = tables.Column("host", verbose_name=_("Host"))
    nova_instance_id = tables.Column("nova_instance_id", verbose_name="Instance ID")

    class Meta(object):
        name = "a10scalinggroupmembertable"
        verbose_name = _("Scaling Group Members")
        table_actions = ()
        row_actions = ()


def get_policy_detail(datum):
    return reverse_lazy(URL_PREFIX + "scalingpolicydetail",
                        kwargs={"scaling_policy_id": datum["id"]})


class A10ScalingPolicyTable(tables.DataTable):
    id = tables.Column("id", verbose_name=_("ID"), hidden=True)
    name = tables.Column("name", verbose_name=("Name"), link=get_policy_detail)
    description = tables.Column("description", verbose_name=_("Description"))
    cooldown = tables.Column("cooldown", verbose_name=_("Cooldown Period"))
    min_instances = tables.Column("min_instances", verbose_name=_("Min. Instances"))
    max_instances = tables.Column("max_instances", verbose_name=_("Max. Instances"))

    class Meta(object):
        name = "a10scalingpolicytable"
        verbose_name = _("Scaling Policies")
        table_actions = (AddScalingPolicyLink, DeleteScalingPolicyAction)
        row_actions = (UpdateScalingPolicyLink, DeleteScalingPolicyAction)


def reaction_summary(alarm, action):
    """Pretty print a reaction in the following format:

    When {agg} {measurement} is {operator} {threshold}...
        {unit} over a period of {period} {period_unit}.
        {action} {amount} instances.
    """

    # TODO(mdurrant) - Insert brains for singular/plurals.
    # If period > 1 period_units
    # If |threshold| > 1 units
    alarm_fmt = "When {0} {1} is {2} {3} {4} over a period of {5} {6}"
    action_fmt = "{0} by {1} instances."
    alarm_txt = alarm_fmt.format(AGG_DICT.get(alarm["aggregation"]),
                                 alarm["measurement"], OP_DICT.get(alarm["operator"]),
                                 alarm["threshold"], UNIT_DICT.get(alarm["unit"]),
                                 alarm["period"], alarm["period_unit"])
    action_txt = action_fmt.format(action["action"], action["amount"])

    return "{0}, {1}".format(alarm_txt, action_txt)


class UpdatePolicyReactionTable(tables.DataTable):
    position = tables.Column("position", verbose_name=_("Position"), hidden=True)
    alarm_id = tables.Column("alarm_id", hidden=True)
    action_id = tables.Column("action_id", hidden=True)
    alarm_name = tables.Column(lambda x: x["alarm"]["name"],
                               verbose_name=_("Alarm"),
                               link=(lambda x: reverse_lazy(URL_PREFIX + "updatealarm",
                                                            kwargs={"id": x["alarm_id"]})))
    action_name = tables.Column(lambda x: x["action"]["name"],
                                verbose_name=_("Action"),
                                link=(lambda x: reverse_lazy(URL_PREFIX + "updateaction",
                                                             kwargs={"id": x["action_id"]})))

    summary = tables.Column(lambda x: reaction_summary(x["alarm"], x["action"]),
                            verbose_name=_("Summary"))

    def get_object_id(self, datum):
        return datum['position']

    class Meta(object):
        name = "updatepolicyreactiontable"
        verbose_name = _("Scaling Policy Reactions")
        table_actions = (AddReactionLink,)
        row_actions = (DeleteReactionLink, )


class A10ScalingAlarmTable(tables.DataTable):
    id = tables.Column("id", verbose_name=_("ID"), hidden=True)
    tenant_id = tables.Column("tenant_id", verbose_name=_("Tenant ID"), hidden=True)
    name = tables.Column("name", verbose_name="Name")
    description = tables.Column("description", verbose_name="Description")
    aggregation = tables.Column("aggregation", verbose_name="Aggregation")
    measurement = tables.Column("measurement", verbose_name="Measurement")
    operator = tables.Column("operator", verbose_name="Operator")
    threshold = tables.Column("threshold", verbose_name="Threshold")
    unit = tables.Column("unit", verbose_name="Unit")
    period = tables.Column("period", verbose_name="Period")
    period_unit = tables.Column("period_unit", verbose_name="period_unit")

    class Meta(object):
        name = "a10scalingalarmtable"
        verbose_name = _("Scaling Alarms")
        table_actions = (AddAlarmLink, DeleteAlarmLink)
        row_actions = (UpdateAlarmLink, DeleteAlarmLink)


class A10ScalingActionTable(tables.DataTable):
    id = tables.Column("id", verbose_name=_("ID"), hidden=True)
    tenant_id = tables.Column("tenant_id", verbose_name=_("Tenant ID"), hidden=True)
    name = tables.Column("name", verbose_name="Name")
    description = tables.Column("description", verbose_name="Description")
    action = tables.Column("action", verbose_name="Action")
    amount = tables.Column("amount", verbose_name="Amount")

    class Meta(object):
        name = "a10scalingactiontable"
        verbose_name = _("Scaling Actions")
        table_actions = (AddActionLink, DeleteActionLink)
        row_actions = (UpdateActionLink, DeleteActionLink)



# Copyright (C) 2014-2016, A10 Networks Inc. All rights reserved.
from django.conf.urls import patterns
from django.conf.urls import url

from a10_horizon.dashboard.a10networks.a10scaling import views

# TODO(mdurrant): Create urls from cross product of nouns/verbs...itertools.product?
urlpatterns = patterns("a10_horizon.dashboard.a10networks.a10scaling.views",
                       url(r'^$',
                           views.IndexView.as_view(),
                           name='index'),
                       url(r'^scalingpolicy/add$',
                           views.AddPolicyView.as_view(),
                           name='addscalingpolicy'),
                       url(r'^scalingpolicy/update/(?P<scaling_policy_id>[^/]*)$',
                           views.UpdatePolicyView.as_view(),
                           name='updatescalingpolicy'),
                       url(r'^scalingaction/add/$',
                           views.AddActionView.as_view(),
                           name='addaction'),
                       url(r'^action/update/(?P<id>[^/]*)$',
                           views.UpdateActionView.as_view(),
                           name='updateaction'),
                       url(r'^alarm/add/$',
                           views.AddAlarmView.as_view(),
                           name='addalarm'),
                       url(r'^alarm/update/(?P<id>[^/]*)$',
                           views.UpdateAlarmView.as_view(),
                           name='updatealarm'),
                       url(r'^scalingpolicy/(?P<scaling_policy_id>[^/]*)/addreaction$',
                           views.AddReactionView.as_view(),
                           name='addreaction'),
                       url(r'^scalingpolicy/(?P<scaling_policy_id>[^/]*)/detail$',
                           views.PolicyDetailView.as_view(),
                           name='scalingpolicydetail'),
                       url(r'^scalinggroup/(?P<scaling_group_id>[^/]*)/detail$',
                           views.GroupDetailView.as_view(),
                           name='scalinggroupdetail'))



# Copyright 2015 A10 Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging
import uuid

from django.utils.translation import ugettext_lazy as _

import horizon.forms as forms
import horizon.tables as tables
import horizon.workflows as workflows
import openstack_dashboard.api.glance as glance_api
import openstack_dashboard.api.neutron as neutron_api
import openstack_dashboard.api.nova as nova_api


GLANCE_API_VERSION_LIST = 2
GLANCE_API_VERSION_CREATE = 2
GLANCE_API_VERSION_UPDATE = 1

LOG = logging.getLogger(__name__)




# Copyright 2015,  A10 Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import tabs

import a10_horizon.dashboard.a10networks.a10appliances.tables as p_tables
import a10_horizon.dashboard.api.a10devices as a10api


class A10AppliancesTab(tabs.TableTab):
    table_classes = (p_tables.A10ApplianceTable,)
    name = _("A10 Appliances")
    slug = "a10appliancestab"
    template_name = "horizon/common/_detail_table.html"
    preload = False

    def get_a10appliancestable_data(self):
        result = []

        try:
            result = a10api.get_a10_appliances(self.request)
        except Exception:
            result = []
            exceptions.handle(self.tab_group.request,
                              _('Unable to retrieve appliance list.'))
        return result


class A10DeviceInstanceTab(tabs.TableTab):
    table_classes = (p_tables.A10DeviceInstanceTable,)
    name = "a10deviceinstancestab"


class A10Tabs(tabs.TabGroup):
    slug = "a10tabs"
    tabs = (A10AppliancesTab, )
    sticky = True



# Copyright (C) 2014-2015, A10 Networks Inc. All rights reserved.

# http://legacy.python.org/dev/peps/pep-0440/#version-scheme
# Use the pep-0440 as a versioning guidline
# There are always four parts, although trailing parts 'may' be empty.
# Idealy the first 3 parts will always have a value
__version__ = "0.1.0"
__version_info__ = ('0', '1', '0')

# from a10_horizon.dashboard import base


# add_tab = base.add_tab
# post_hack = base.post_hack



# Copyright (C) 2014-2016, A10 Networks Inc. All rights reserved.

import logging

from django.core.urlresolvers import reverse_lazy
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import messages

# from openstack_dashboard import api
from a10_horizon.dashboard.api import scaling as api
from a10_neutronclient.resources import a10_scaling_group as scaling_resources


LOG = logging.getLogger(__name__)


def array_to_choices(choices=[]):
    return map(lambda x: (x, x), choices)


class UpdatePolicy(forms.SelfHandlingForm):
    def __init__(self, *args, **kwargs):
        super(UpdatePolicy, self).__init__(*args, **kwargs)
        self.submit_url = kwargs.get("id")

    id = forms.CharField(label=_("ID"), widget=forms.TextInput(attrs={'readonly': 'readonly'}))
    name = forms.CharField(label=_("Name"), min_length=1, max_length=255,
                           required=True)
    description = forms.CharField(label=_("Description"), min_length=1,
                                  max_length=255, required=False)
    # TODO(mdurrant) - Get min/max cooldown/instances from config?
    cooldown = forms.IntegerField(label=_("Cooldown Period in Seconds"),
                                  min_value=1,
                                  max_value=65536,
                                  required=True)
    min_instances = forms.IntegerField(label=_("Minimum Instances"),
                                       min_value=1,
                                       required=True)
    max_instances = forms.IntegerField(label=_("Maximum Instances"),
                                       required=False)

    failure_url = "horizon:project:a10scaling:index"
    success_url = "horizon:project:a10scaling:index"
    # redirect_url = reverse_lazy('horizon:project:a10scaling:updatescalingpolicy')

    def handle(self, request, context):
        try:
            policy = api.update_a10_scaling_policy(request, **context)
            msg = _("Scaling Policy {0} was successfully updated").format(context["name"])
            messages.success(request, msg)
            return policy
        except Exception as ex:
            msg = _("Failed to update Scaling Policy %s") % context["name"]
            LOG.exception(ex)
            redirect = reverse_lazy(self.failure_url)
            exceptions.handle(request, msg, redirect=redirect)


class UpdateAlarm(forms.SelfHandlingForm):
    id = forms.CharField(label=_("ID"), widget=forms.TextInput(attrs={'readonly': 'readonly'}))
    name = forms.CharField(label=_("Name"), min_length=1, max_length=255,
                           required=True)
    description = forms.CharField(label=_("Description"), min_length=1,
                                  max_length=255, required=False)
    aggregation = forms.ChoiceField(label=_("Aggregation"),
                                    choices=array_to_choices(scaling_resources.ALARM_AGGREGATIONS),
                                    required=True)
    measurement = forms.ChoiceField(label=_("Measurement"),
                                    choices=array_to_choices(scaling_resources.ALARM_MEASUREMENTS),
                                    required=True)
    operator = forms.ChoiceField(label=_("Operator"),
                                 choices=array_to_choices(scaling_resources.ALARM_OPERATORS),
                                 required=True,)
    threshold = forms.FloatField(label=_("Threshold"),
                                 required=True)
    unit = forms.DynamicChoiceField(label=_("Unit"),
                                    choices=array_to_choices(scaling_resources.ALARM_UNITS),
                                    required=True)
    period = forms.IntegerField(label=_("Period"),
                                required=True)
    period_unit = forms.ChoiceField(label=_("Period Unit"),
                                    choices=array_to_choices(scaling_resources.ALARM_PERIOD_UNITS),
                                    required=True)

    failure_url = "horizon:project:a10scaling:index"
    success_url = "horizon:project:a10scaling:index"

    def handle(self, request, context):
        try:
            alarm = api.update_a10_scaling_alarm(request, **context)
            msg = _("Scaling Alarm {0} was successfully updated").format(context["name"])
            messages.success(request, msg)
            return alarm
        except Exception as ex:
            msg = _("Failed to update Scaling Action")
            LOG.exception(ex)
            redirect = reverse_lazy(self.failure_url)
            exceptions.handle(request, msg, redirect=redirect)


class UpdateAction(forms.SelfHandlingForm):
    id = forms.CharField(label=_("ID"), widget=forms.TextInput(attrs={'readonly': 'readonly'}))
    name = forms.CharField(label=_("Name"), min_length=1, max_length=255,
                           required=True)
    description = forms.CharField(label=_("Description"), min_length=1,
                                  max_length=255, required=False)
    action = forms.ChoiceField(label=_("Action"),
                               choices=array_to_choices(scaling_resources.ACTIONS),
                               required=True)
    amount = forms.IntegerField(label=_("Amount"), required=True)

    failure_url = "horizon:project:a10scaling:index"
    success_url = "horizon:project:a10scaling:index"

    def handle(self, request, context):
        try:
            action = api.update_a10_scaling_action(request, **context)
            msg = _("Scaling Action was successfully updated")
            messages.success(request, msg)
            return action
        except Exception as ex:
            msg = _("Failed to update Scaling Action")
            LOG.exception(ex)
            redirect = reverse_lazy(self.failure_url)
            exceptions.handle(request, msg, redirect=redirect)
